# -*- coding: utf-8 -*-
# Copyright 2010 Thamini S.à.R.L    This software is licensed under the
# GNU Affero General Public License version 3 (see the file LICENSE).

from osv import osv
from osv import fields
from tools.translate import _
import string

class advance_structured_content_block(osv.osv):
    _name = 'structured.content.block'
advance_structured_content_block()

class advance_structured_content_block_list(osv.osv):
    _name = 'structured.content.block.list'
advance_structured_content_block_list()

class advance_structured_content_block_list_elem(osv.osv):
    _name = 'structured.content.block.list.elem'
advance_structured_content_block_list_elem()

class advance_structured_content_block_accordion(osv.osv):
    _name = 'structured.content.block.accordion'
advance_structured_content_block_accordion()

class advance_structured_content_block_accordion_elem(osv.osv):
    _name = 'structured.content.block.accordion.elem'
advance_structured_content_block_accordion_elem()

_LIST_TYPES = [
    ('#', 'Auto.'),
    ('decimal', 'Decimal'),
    ('none', 'None'),
    ('disc', 'Disc'),
    ('upper_alpha', 'Upper Alpha.'),
    ('lower_alpha', 'Lower Alpha.'),
    ('dash', 'Dash'),
    ('h1', 'Title 1'),
    ('h2', 'Title 2'),
]

_BLOCK_TYPES = [
    ('shorttext', 'Short Text'),
    ('text', 'Text'),
    ('list', 'List'),
    ('url', 'url'),
    ('accordion', 'Accordion'),
    ('binary', 'Binary Attachment'),
    ('dms', 'DMS Attachment'),
    ('hr', 'Horizontal Line'),
]

class structured_content_type(osv.osv):
    _name = 'structured.content.type'
    _columns = {
        'code': fields.char('Code', size=32, required=True),
        'name': fields.char('Name', size=32, required=True),
        'basetype': fields.selection(_BLOCK_TYPES, 'Base Type', required=True, size=16),
    }

    _defaults = {
        'basetype': lambda *a: 'text',
    }
structured_content_type()

class structured_content_page_type(osv.osv):
    _name = 'structured.content.page.type'
    _columns = {
        'code': fields.char('Code', size=32, select=1),
        'name': fields.char('Name', size=64, select=1),
    }
structured_content_page_type()

class structured_content_page(osv.osv):
    _name = 'structured.content.page'
    _order = 'sequence'
    _columns = {
        # 'res_id': fields.many2one(YOU_OBJECT, required=True)
        'sequence': fields.integer('Number', required=True),
        'name': fields.char('Name', size=64),
        'type_id': fields.many2one('structured.content.page.type', 'Type'),
        'block_ids': fields.one2many('structured.content.block', 'page_id', 'Blocks'),
    }

    _defaults = {
        'sequence': lambda *a: 1,
    }

    def onchange_type_id(self, cr, uid, ids, type_id, name, context=None):
        ocv = {'value': {}}
        if name:
            return ocv
        ocv['value']['name'] = ''
        if type_id:
            type_pool = self.pool.get('structured.content.page.type')
            type_name = type_pool.name_get(cr, uid, [type_id], context=context)
            if type_name:
                ocv['value']['name'] = type_name[0][1]
        return ocv

structured_content_page()

class structured_content_block(osv.osv):
    _name = 'structured.content.block'
    _rec_name = 'computed_name'
    _order = 'sequence'

    def _computed_name(self, cr, uid, ids, fn, args, context=None):
        ira_pool = self.pool.get('ir.attachment')
        res = dict.fromkeys(ids, '')

        for block in self.browse(cr, uid, ids, context=context):
            basetype = block.basetype
            r = ''
            if basetype == 'shorttext':
                r = block.shorttext
            elif basetype == 'text':
                r = block.text
            elif basetype == 'url':
                r = block.shorttext
                if block.text:
                    r += ' (%s)' % (block.text)
            elif basetype == 'binary':
                pass # TODO: for binary who to represent that ?
            elif basetype == 'dms':
                if block.content_attach_id:
                    ir_record = ira_pool.browse(cr, uid, block.content_attach_id.id, context=context)
                    r = ir_record.name
            elif basetype == 'list':
                pass # TODO: generate list name_get()
                vals = []
                for k in block.list_ids:
                    v = k.generate_preview()
                    if v:
                        vals.append(v.get(k.id, ''))
                r = '\n'.join(vals)
            elif basetype == 'accordion':
                pass # TODO: generate accordion name_get()
            elif basetype == 'hr':
                r = u'-' * 64
            res[block.id] = r
        return res

    def _get_types(self, cr, uid, context=None):
        res = []
        type_pool = self.pool.get('structured.content.type')
        type_ids = type_pool.search(cr, 1, [], context=context)
        for type_ in type_pool.browse(cr, 1, type_ids, context=context):
            res.append((type_.code, type_.name))
        return res

    def _get_basetype(self, cr, uid, ids, fn, args, context=None):
        res = dict.fromkeys(ids, 'shorttext')
        type_pool = self.pool.get('structured.content.type')
        type_ids = type_pool.search(cr, uid, [], context=context)
        type_data = type_pool.read(cr, uid, type_ids, ['code', 'basetype'], context=context)
        type_map = dict([ (x['code'], x['basetype']) for x in type_data ])

        for block in self.browse(cr, uid, ids, context=context):
            basetype = type_map[block.type]
            res[block.id] = basetype
        return res

    _columns = {
        'page_id': fields.many2one('structured.content.page', 'Page', required=True, ondelete='cascade'),
        'sequence': fields.integer('Position', required=True),
#        'type_id': fields.many2one('structured.content.type', 'Type', required=True),
        'type': fields.selection(_get_types, 'Type', required=True),
#        'basetype': fields.related('type_id', 'basetype', type='selection', selection=_BLOCK_TYPES, string='Base Type', readonly=True, size=16),
        'basetype': fields.function(_get_basetype, type='selection', method=True, selection=_BLOCK_TYPES, string='Base Type', size=16),
        'computed_name': fields.function(_computed_name, type='text', method=True, readonly=True, string='Name'),
        'shorttext': fields.char('Short Text', size=255),
        'url': fields.text('Url'),
        'text': fields.text('Text'),
        'content_binary': fields.binary('Content'),
        'content_attach_id': fields.many2one('ir.attachment', 'Content from DMS'),
        'list_ids': fields.one2many('structured.content.block.list', 'block_id', 'Lists'),
        'accordion_ids': fields.one2many('structured.content.block.accordion', 'block_id', 'Accordions'),
    }

    def onchange_type(self, cr, uid, ids, type_val, context=None):
        # TODO
        ocv = {'value': {'basetype': False}}
        if not type_val:
            return ocv
        type_pool = self.pool.get('structured.content.type')
        type_ids = type_pool.search(cr, uid, [('code','=',type_val)], context=context)
        if type_ids:
            type_ = type_pool.browse(cr, uid, type_ids[0], context=context)
            ocv['value']['basetype'] = type_.basetype
        return ocv

structured_content_block()

class structured_content_block_list(osv.osv):
    _name = 'structured.content.block.list'
    _columns = {
        'block_id': fields.many2one('structured.content.block', 'Block', required=True, ondelete='cascade'),
        'name': fields.char('Name', size=255, required=True),
        'elem_ids': fields.one2many('structured.content.block.list.elem', 'list_id', 'Elements'),
    }

    def get_sequence(self, level, type):
        class cgen(object):
            def __init__(self, level, type):
                self.level = level
                self.type = type
                self.i = 0

            def next(self, data):
                r = None
                if self.type == 'none':
                    r = data
                elif self.type == 'dash':
                    r = u'- %s' % (data)
                elif self.type == 'h1':
                    r = u'%s' % (data.upper())
                elif self.type == 'h2':
                    r = u'%s' % (data.capitalize())
                elif self.type == 'decimal':
                    r = u'%d. %s' % (self.i + 1, data)
                elif self.type == 'disc':
                    r = u'• %s' % (data)
                elif self.type == 'upper_alpha':
                    r = u'%s. %s' % (string.uppercase[self.i % 26], data)
                elif self.type == 'lower_alpha':
                    r = u'%s. %s' % (string.lowercase[self.i % 26], data)
                self.i += 1
                return r

            def reset(self):
                self.i = 0
        return cgen(level, type)

    def generate_preview(self, cr, uid, ids, context=None):
        res = dict.fromkeys(ids, '')
        level_pool = self.pool.get('structured.content.block.list.level')
        level_pool_ids = level_pool.search(cr, uid, [], context=context)
        levels_list = level_pool.browse(cr, uid, level_pool_ids, context=context)
        levels = dict([(x.code, i) for i, x in enumerate(levels_list) ])
        sequences = {}
        # XXX: check if there is not levels!!!
        for blocklist in self.browse(cr, uid, ids, context=None):
            lst = []
            ln = None # global level number
            lt = None # global type
            lseqs = []
            for e in blocklist.elem_ids:
                indent_level = len(lseqs)+1
                if e.level == '#':
                    if ln is None:
                        eln = levels[levels_list[0].code]
                    else:
                        eln = ln # automatiq mode
                else:
                    eln = levels[e.level] # elem level number

                if e.type == '#':
                    if lt is None:
                        lt = 'decimal'
                    elt = lt
                else:
                    if lt is None:
                        lt = e.type
                    elt = e.type

                if ln is None or eln > ln:
                    # XXX: manager case: ln is None and e.level == '#'
                    ln = eln
                    lseqs.append(self.get_sequence(eln, elt))
                elif eln < ln:
                    while True:
                        if not lseqs:
                            lseqs.append(self.get_sequence(eln, elt))
                            ln = eln
                            break
                        if lseqs[-1].level < eln:
                            lseqs.append(self.get_sequence(eln, elt))
                            ln = eln
                            break # gone too far
                        if lseqs[-1].level == eln or lseqs[-1].level == '#':
                            break
                        oseq = lseqs.pop()
                        ln = lseqs[-1].level
                lst.append((lseqs[-1].next(e.text), eln))
            r = u'\n'.join([u' '*4*l[1]+l[0] for l in lst ])
            res[blocklist.id] = r
        return res
structured_content_block_list()

class structured_content_block_list_level(osv.osv):
    _name = 'structured.content.block.list.level'
    _order = 'sequence ASC'
    _columns = {
        'sequence': fields.integer('Sequence', required=True),
        'code': fields.char('Code', size=32, required=True),
        'name': fields.char('Name', size=32, required=True, translate=True),
        'default_type': fields.selection(_LIST_TYPES, 'Type'),
    }
    _defaults = {
        'sequence': lambda *a: 0,
    }
structured_content_block_list_level()

class structured_content_block_list_elem(osv.osv):
    _name = 'structured.content.block.list.elem'
    _rec_name = 'text'
    _order = 'sequence ASC, id ASC'

    def _get_levels(self, cr, uid, context=None):
        res = [('#', _('Auto'))]
        # run 'admin' to avoid giving un-necessary rights to user only
        # to fill the selection
        type_pool = self.pool.get('structured.content.block.list.level')
        tids = type_pool.search(cr, 1, [], context=context)
        for r in type_pool.browse(cr, 1, tids, context=context):
            res.append((r.code, r.name))
        return res

    _columns = {
        'list_id': fields.many2one('structured.content.block.list', 'List', required=True, ondelete='cascade'),
        'sequence': fields.integer('Position', required=True),
        'level': fields.selection(_get_levels, 'Levels', size=32, required=True),
        'type': fields.selection(_LIST_TYPES, 'Type', required=True),
        'text': fields.text('Text'),
    }

    _defaults = {
        'sequence': lambda *a: 0,
        'level': lambda *a: '#',
        'type': lambda *a: '#',
    }

    def onchange_level_id(self, cr, uid, ids, level, context=None):
        ocv = {'value': {}}
        if not level:
            return ocv
        level_pool = self.pool.get('structured.content.block.list.level')
        level_ids = level_pool.search(cr, uid, [('code','=',level)], context=context)
        if level_ids:
            level =  level_pool.browse(cr, uid, level_ids[0], context=context)
            if level.default_type:
                ocv['value']['type'] = level.default_type
        return ocv

structured_content_block_list_elem()

class structured_content_block_accordion(osv.osv):
    _name = 'structured.content.block.accordion'
    _rec_name = 'elem_ids'
    _columns = {
        'block_id': fields.many2one('structured.content.block', 'Block', required=True, ondelete='cascade'),
        'elem_ids': fields.one2many('structured.content.block.accordion.elem', 'accordion_id', 'Elements'),
    }
structured_content_block_accordion()

class structured_content_block_accordion_elem(osv.osv):
    _name = 'structured.content.block.accordion.elem'
    _rec_name = 'title'
    _order = 'sequence ASC, id ASC'

    def _get_reference_type(self, cr, uid, context=None):
        return [('', '')]

    def onchange_reference(self, cr, uid, ids, reference, context=None):
        ocv = {'value': {'title': ''}}
        # check if reference fit fields.reference values (res_model,res_id)
        if not reference or (reference and ',' not in reference):
            return ocv
        model, id = reference.split(',', 1)
        id = int(id)
        # check if model or id are not NULL
        if not model or id == 0:
            return ocv
        model_pool = self.pool.get(model)
        name = model_pool.name_get(cr, uid, [id], context=context)[0][1]
        ocv['value']['title'] = name
        return ocv

    _columns = {
        'accordion_id': fields.many2one('structured.content.block.accordion', 'Accordion', ondelete='cascade'),
        'sequence': fields.integer('Position', required=True),
        'reference': fields.reference('Reference', lambda self, cr, uid, context: self._get_reference_type(cr, uid, context=context), size=128),
        'title': fields.char('Title', size=255, required=True),
        'description': fields.text('Description'),
        'link_alt': fields.char('Link Name', size=255),
    }
structured_content_block_accordion_elem()
